package dev.fyloz.trial.colorrecipesexplorer.rest

import dev.fyloz.trial.colorrecipesexplorer.model.*
import dev.fyloz.trial.colorrecipesexplorer.service.MixService
import dev.fyloz.trial.colorrecipesexplorer.service.RecipeService
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import java.net.URI
import javax.validation.Valid


private const val RECIPE_CONTROLLER_PATH = "api/recipe"
private const val MIX_CONTROLLER_PATH = "api/recipe/mix"

@RestController
@RequestMapping(RECIPE_CONTROLLER_PATH)
class RecipeController(recipeService: RecipeService) :
    AbstractModelRestApiController<Recipe, RecipeSaveDto, RecipeUpdateDto, RecipeService>(
        recipeService,
        RECIPE_CONTROLLER_PATH
    ) {
    @PutMapping("public")
    fun updatePublicData(@Valid @RequestBody publicDataDto: RecipePublicDataDto): ResponseEntity<Void> {
        service.updatePublicData(publicDataDto)
        return ResponseEntity.noContent().build()
    }
}

@RestController
@RequestMapping(MIX_CONTROLLER_PATH)
class MixController(val mixService: MixService) {
    @PostMapping
    fun save(@Valid @RequestBody entity: MixSaveDto): ResponseEntity<Mix> {
        val saved = mixService.save(entity)
        return ResponseEntity
            .created(URI("$MIX_CONTROLLER_PATH/${saved.id}"))
            .body(saved)
    }

    @PutMapping
    fun update(@Valid @RequestBody entity: MixUpdateDto): ResponseEntity<Void> {
        mixService.update(entity)
        return ResponseEntity
            .noContent()
            .build()
    }
}
